* Output options - page and line size, no page breaks, debug macros;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=100 formdlim='-';										
* Kill default titles on output; TITLE;
* Kill names of dataset in log; ODS TRACE OFF;

* Location for files to be saved - CHANGE THIS;
%LET filesave=F:\Example Data\Chapter 15 Data;
LIBNAME filesave "&filesave.";

* Defining global variables to be replaced in code below;
%GLOBAL datafile mixoptions ddfmoption classvar; 
* Data file name;		%LET datafile=chapter15;
* MIXED options;		%LET mixoptions=COVTEST	METHOD=REML NOCLPRINT NOITPRINT NAMELEN=100 MAXITER=500;
* DDFM specification;	%LET ddfmoption=Satterthwaite;
* In CLASS statements;	%LET classvar= ID item older;

* Copying data to work library, centering predictors;
DATA &datafile.; SET filesave.&datafile.; 
	* Creating natural log transform of RT;
		lg_rt = LOG(rt_sec);
	* Recoding age into two variables (missing, younger, older);
	* 'agegroup' will be treated as continuous, 'older' will be categorical;
		IF age=. THEN agegroup=.;
			ELSE IF age LE 40 THEN DO; agegroup=0; older=0; END;
			ELSE IF age GT 40 THEN DO; agegroup=1; older=1; END;
		IF agegroup=. THEN yrs65=.;
			ELSE IF agegroup=0 THEN yrs65=0; 
			ELSE IF agegroup=1 THEN yrs65=age-65;
	* Centering meaning(relevance) and salience at midpoint;
		rel3 = meaning - 3; *meaning=relevance;
		sal3 = salience - 3; 
	* Labeling new variables;
		LABEL rt_sec=	"rt_sec: Response Time in Seconds"
			  lg_rt=	"lg_rt: Natural Log RT in Seconds"
		  	  agegroup=	"agegroup: Age Group (0=young, 1=old)"
			  older=	"older: Age Group (0=young, 1=old)"
		  	  yrs65=	"yrs65: Years Over Age 65 (0=65)"
		  	  rel3=		"rel3: Centered Meaning/Relevance (0=3)"
		  	  sal3=		"sal3: Centered Salience (0=3)"; 
RUN;
PROC MEANS DATA=&datafile.; CLASS agegroup; VAR age; RUN;

*** UNCONDITIONAL MODELS;

TITLE "Eq 15.8: Empty Means, E-only";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt=  / SOLUTION DDFM=&ddfmoption.;										
run;
TITLE "Eq 15.9: Empty Means, Subject Random Intercept";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt=  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept / TYPE=UN SUBJECT=ID &grouprand.;	
run;
TITLE "Eq 15.10: Empty Means, Crossed Subjects and Items";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt=  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;	
run;

*** CONDITIONAL MODELS;

TITLE "Eq 15.11: Add Item Predictors";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;	
run;
TITLE "Eq 15.12: Add Subject Predictors";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;	
run;
TITLE "Eq 15.13a: Add Subject Random Salience Slope";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept sal3 / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;	
run;
TITLE "Eq 15.13b: Add Subject Random Meaning Slope";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  / SOLUTION DDFM=&ddfmoption.;
	RANDOM Intercept sal3 rel3 / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;	
run;
TITLE "Eq 15.14a: All Possible Interactions";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  
				  rel3*agegroup rel3*yrs65 sal3*agegroup sal3*yrs65
				  rel3*sal3*agegroup rel3*sal3*yrs65 / SOLUTION DDFM=&ddfmoption.;										
  	RANDOM Intercept sal3 / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;
run;
TITLE "Eq 15.14b: All Significant Interactions";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  
				  rel3*agegroup sal3*agegroup
				  rel3*sal3*agegroup / SOLUTION DDFM=&ddfmoption.;										
  	RANDOM Intercept sal3 / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;
run;

*** HETEROGENEOUS VARIANCE MODELS;

TITLE "Eq 15.15a: Add Separate Subject Residual Variances by Age Group ";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  
				  rel3*agegroup sal3*agegroup
				  rel3*sal3*agegroup / SOLUTION DDFM=&ddfmoption.;										
  	RANDOM Intercept sal3 / TYPE=UN SUBJECT=ID;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;
	REPEATED / TYPE=VC SUBJECT=ID GROUP=older;
run;

*** FINAL MODEL;

TITLE "Eq 15.15b: Add Separate Subject Random Variances by Age Group";																					
PROC MIXED DATA=&datafile. &mixoptions.;
	CLASS &classvar.;
  	MODEL lg_rt= rel3 sal3 rel3*sal3 agegroup yrs65  
				  rel3*agegroup sal3*agegroup
				  rel3*sal3*agegroup / SOLUTION DDFM=&ddfmoption. OUTPM=Conditional;										
  	RANDOM Intercept sal3 / TYPE=UN SUBJECT=ID GROUP=older;
	RANDOM Intercept / TYPE=UN SUBJECT=Item;
	REPEATED / TYPE=VC SUBJECT=ID GROUP=older;
	ESTIMATE "Relevance Effect at Salience=3 for Younger"	rel3 1 rel3*agegroup 0 rel3*sal3 0 rel3*sal3*agegroup 0;
	ESTIMATE "Relevance Effect at Salience=3 for Older"		rel3 1 rel3*agegroup 1 rel3*sal3 0 rel3*sal3*agegroup 0;
	ESTIMATE "Salience Effect at Relevance=3 for Younger"	sal3 1 sal3*agegroup 0 rel3*sal3 0 rel3*sal3*agegroup 0;
	ESTIMATE "Salience Effect at Relevance=3 for Older"		sal3 1 sal3*agegroup 1 rel3*sal3 0 rel3*sal3*agegroup 0;
	ESTIMATE "Relevance by Salience for Younger"  			rel3*sal3 1 rel3*sal3*agegroup 0;
	ESTIMATE "Relevance by Salience for Older"  			rel3*sal3 1 rel3*sal3*agegroup 1;
run;
PROC CORR DATA=Conditional; VAR pred lg_rt; run;
